import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:calculus/common/ui/parameters/parameter_painter.dart';

import 'package:calculus/config/default_activity_settings.dart';
import 'package:calculus/config/default_global_settings.dart';
import 'package:calculus/models/settings/settings_activity.dart';
import 'package:calculus/models/settings/settings_global.dart';

class ParameterWidget extends StatelessWidget {
  const ParameterWidget({
    super.key,
    required this.code,
    required this.value,
    required this.isSelected,
    required this.size,
    required this.activitySettings,
    required this.globalSettings,
    required this.onPressed,
  });

  final String code;
  final String value;
  final bool isSelected;
  final double size;
  final ActivitySettings activitySettings;
  final GlobalSettings globalSettings;
  final VoidCallback onPressed;

  static const Color buttonColorActive = Colors.blue;
  static const Color buttonColorInactive = Colors.white;
  static const double buttonBorderWidth = 4.0;
  static const double buttonBorderRadius = 12.0;

  @override
  Widget build(BuildContext context) {
    Widget content = const SizedBox.shrink();

    switch (code) {
      case DefaultActivitySettings.parameterCodeLevel:
        content = getLevelParameterItem();
        break;
      case DefaultGlobalSettings.parameterCodeSkin:
        content = getSkinParameterItem();
        break;
      default:
        printlog('Unknown parameter: $code/$value');
        content = getUnknownParameterItem();
    }

    final Color buttonColor = isSelected ? buttonColorActive : buttonColorInactive;

    return Container(
      decoration: BoxDecoration(
        color: buttonColor,
        borderRadius: BorderRadius.circular(buttonBorderRadius),
        border: Border.all(
          color: buttonColor,
          width: buttonBorderWidth,
        ),
      ),
      child: content,
    );
  }

  // "unknown" parameter -> simple block with text
  Widget getUnknownParameterItem() {
    return StyledButton.text(
      caption: '$code / $value',
      color: Colors.grey,
      onPressed: null,
    );
  }

  Widget getLevelParameterItem() {
    Color backgroundColor = Colors.grey;

    switch (value) {
      case DefaultActivitySettings.levelValueEasy:
        backgroundColor = Colors.green;
        break;
      case DefaultActivitySettings.levelValueMedium:
        backgroundColor = Colors.orange;
        break;
      case DefaultActivitySettings.levelValueHard:
        backgroundColor = Colors.red;
        break;
      case DefaultActivitySettings.levelValueNightmare:
        backgroundColor = Colors.purple;
        break;
      default:
        printlog('Wrong value for level parameter value: $value');
    }

    return StyledButton(
      color: backgroundColor,
      onPressed: onPressed,
      child: CustomPaint(
        size: Size(size, size),
        willChange: false,
        painter: ParameterPainter(
          code: code,
          value: value,
          activitySettings: activitySettings,
          globalSettings: globalSettings,
        ),
        isComplex: true,
      ),
    );
  }

  Widget getSkinParameterItem() {
    return StyledButton(
      color: Colors.green.shade800,
      onPressed: onPressed,
      child: Image(
        image: AssetImage('assets/ui/${DefaultGlobalSettings.parameterCodeSkin}_$value.png'),
        fit: BoxFit.fill,
      ),
    );
  }
}
